@@ -14,14 +14,13 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
     vinv <- par[3]
     if (isreg) xcoef <- matrix(par[4:length(par)],ncol=1)
     
-    if (tau2 < 0.0) return(NA)
+    if (tau2 < 0.0) return(NA)    
+    if (vinv < 0.0) return(NA)
+    
     
     f <- function(nu,oney,onesigma2,onemods,muhat,tau2,vinv,xcoef) {
-      if ((tau2==0.0)|(vinv<0.0)) onel <- rep(0.0,length(nu))
-      else {
-        if (isreg) onel <- exp(-(oney-muhat-matrix(onemods,nrow=1) %*% xcoef-nu)^2/(2*onesigma2))*(1.0/sqrt(tau2))*dt(nu/sqrt(tau2),df=1.0/vinv)
+         if (isreg) onel <- exp(-(oney-muhat-matrix(onemods,nrow=1) %*% xcoef-nu)^2/(2*onesigma2))*(1.0/sqrt(tau2))*dt(nu/sqrt(tau2),df=1.0/vinv)
         else onel <- exp(-(oney-muhat-nu)^2/(2*onesigma2))*(1.0/sqrt(tau2))*dt(nu/sqrt(tau2),df=1.0/vinv)
-      }
       return(onel)        
     }
     calconell <- function(x) {
@@ -29,7 +28,7 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
       onesigma2 <- x[2]
       if (isreg) onemods <- x[3:length(x)]
       else onemods <- NULL
-       theint <- tryCatch({
+        theint <- tryCatch({
           if (isreg) integrate(f,-Inf,Inf,oney=oney,onesigma2=onesigma2,onemods=onemods,muhat=muhat,tau2=tau2,
                              vinv=vinv,xcoef=xcoef,subdivisions = 300L)$value
         else integrate(f,-Inf,Inf,oney=oney,onesigma2=onesigma2,onemods=NULL,muhat=muhat,tau2=tau2,vinv=vinv,
@@ -42,7 +41,7 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
       ll <- log(theint)
       return(ll)
     }
-    if (vinv==0.0){   
+    if ((vinv==0.0) | (tau2==0.0)){   
       w <- 1.0/(tau2+sei^2)
       if (isreg) negll <- 0.5*sum(log(2*pi)+log(1/w)+w*(yi-muhat-as.vector(mods %*% xcoef))^2)
       else negll <- 0.5*sum(log(2*pi)+log(1/w)+w*(yi-muhat)^2)
@@ -70,14 +69,15 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
   parnames(ll.profilet) <- thenames
   names(start.val) <- thenames
   
-  # vinv=0.0 is a special case for some reason, maybe problem in nlminb?
-  maxfit <- profilenorm.metaplus(yi=yi,sei=sei,mods=mods,justfit=TRUE,plotci=FALSE,slab=NULL)$fittedmodel
-  if (isreg) maxfit@coef <- c(maxfit@coef[1:2],0.0,maxfit@coef[3:length(maxfit@coef)])
-  else maxfit@coef <- c(maxfit@coef[1:2],0.0)
-  names(maxfit@coef)[3] <- "vinv"
-#  print(logLik(maxfit))
-  maxll <- logLik(maxfit)
-  for (vinv in c(0.01,0.05,0.1,0.2,0.5,1)) { 
+#   # vinv=0.0 is a special case for some reason, maybe problem in nlminb?
+#   maxfit <- profilenorm.metaplus(yi=yi,sei=sei,mods=mods,justfit=TRUE,plotci=FALSE,slab=NULL)$fittedmodel
+#   if (isreg) maxfit@coef <- c(maxfit@coef[1:2],0.0,maxfit@coef[3:length(maxfit@coef)])
+#   else maxfit@coef <- c(maxfit@coef[1:2],0.0)
+#   names(maxfit@coef)[3] <- "vinv"
+# #  print(logLik(maxfit))
+#   maxll <- logLik(maxfit)
+maxll <- -Inf
+  for (vinv in c(0.0,0.01,0.05,0.1,0.2,0.5,1)) { 
     start.val[3] <- vinv
     if (isreg) profilet.fit <- suppressWarnings(mle2(ll.profilet,start=start.val,vecpar=TRUE,optimizer="nlminb",
                                                      data=list(yi=yi,sei=sei,mods=mods),
@@ -111,10 +111,20 @@ profilet.metaplus <- function(yi,sei,mods=NULL,justfit=FALSE,plotci=FALSE,slab=N
                                                control=list(eval.max=1000),
                                                  lower=newlower.val,
                                                fixed=list(tau2=0.0,vinv=0.0)))
-  if(logLik(profilet.fit2) > logLik(profilet.fit)){
-    profilet.fit@coef <- profilet.fit2@coef  
+  if(logLik(profilet.fit2) >= logLik(profilet.fit)){
+    profilet.fit@coef <- coef(profilet.fit2)
     profilet.fit@min <- profilet.fit2@min
-  }
+    profilet.fit@details$convergence <- profilet.fit2@details$convergence    
+    profilet.fit@details$message <- profilet.fit2@details$message
+  } 
+  if (profilet.fit@details$convergence!=0) {
+# ignore singularity if due to small vinv
+    if (!(((profilet.fit@details$message=="singular convergence (7)") & (profilet.fit@coef[3]<1e-6)) |
+      (profilet.fit@details$message=="false convergence (8)") |
+        (profilet.fit@details$message=="iteration limit reached without convergence (10)"))) {
+      warning(paste("convergence failed: ",profilet.fit@details$message,sep=""))
+    }
+  } 
   results <- profilet.fit@coef
   if (!justfit) {
     if (isreg) thehessian <- hessian(ll.profilet,results,yi=yi,sei=sei,mods=mods)
